#!/usr/bin/env python
import time
import re
import os
import sys
import signal
import glob
import parser_metadata_files
import video_convert
from optparse import OptionParser

import traceback


if '__main__' == __name__:
    import sys

    signal.signal(signal.SIGINT, signal.SIG_IGN)
    signal.signal(signal.SIGQUIT, signal.SIG_IGN)

    parser = OptionParser()
    parser.add_option("-f", "--folder", type="string", dest="folder", help="Convert folder", default="")
    parser.add_option("-m", "--mode", type="int", dest="mode", help="Convert merge mode, \
        [0: txt merge A/V(Default); 1: uid merge A/V; 2: uid merge audio; 3: uid merge video]", default=0)
    parser.add_option("-p", "--fps", type="int", dest="fps", help="Convert fps, default 15", default=15)
    parser.add_option("-s", "--saving", action="store_true", dest="saving", help="Convert Do not time sync",
                      default=False)
    parser.add_option("-r", "--resolution", type="int", dest="resolution", nargs=2,
                      help="Specific resolution to convert '-r width height' \nEg:'-r 640 360'", default=(0, 0))

    (options, args) = parser.parse_args()
    if not options.folder:
        parser.print_help()
        parser.error("Not set folder")

    try:
        os.environ['FPSARG'] = "%s" % options.fps
        parser_metadata_files.cmds_parse(["dispose", options.folder])
        video_convert.do_work()
        parser_metadata_files.cmds_parse(["clean", options.folder])
    except Exception as e:
        traceback.print_exc()

    